/* BdfServer - Copyright (c) 2009-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.balayage;

import fr.exemole.bdfserver.api.balayage.Balayage;
import fr.exemole.bdfserver.api.balayage.BalayageConstants;
import fr.exemole.bdfserver.api.balayage.BalayageMode;
import fr.exemole.bdfserver.api.balayage.BalayageUnit;
import fr.exemole.bdfserver.api.balayage.SiteMapOption;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.utils.SelectionUtils;
import net.mapeadores.util.localisation.LangContext;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.DocumentFragmentHolder;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class BalayageBuilder {

    private final static short RESOLVED_ALL = 1;
    private final static short RESOLVED_SOME = 2;
    private final static short RESOLVED_NONE = 3;
    private final List<FicheQuery> globalFicheQuery = new ArrayList<FicheQuery>();
    private final Map<String, InternalBalayageMode> modeMap = new LinkedHashMap<String, InternalBalayageMode>();
    private final List<BalayageUnitBuilder> unitList = new ArrayList<BalayageUnitBuilder>();
    private final IncludeCatalogBuilder includeCatalogBuilder = new IncludeCatalogBuilder();
    private final List<String> externalScriptNameList = new ArrayList<String>();
    private final List<String> scrutariExportNameList = new ArrayList<String>();
    private final List<String> sqlExportNameList = new ArrayList<String>();
    private final String name;
    private short defaultLangOption = BalayageConstants.LANG_OPTION_NONE;
    private LangContext langContext;
    private boolean withTransformation;
    private String rootName;
    private String path;
    private SiteMapOption siteMapOption;

    public BalayageBuilder(String name) {
        this.name = name;
    }


    public BalayageBuilder addGlobalFicheQuery(FicheQuery ficheQuery) {
        globalFicheQuery.add(ficheQuery);
        return this;
    }

    public BalayageBuilder setRootName(String rootName) {
        this.rootName = rootName;
        return this;
    }

    public BalayageBuilder setPath(String path) {
        this.path = path;
        return this;
    }

    public BalayageBuilder setWithTransformation(boolean withTransformation) {
        this.withTransformation = withTransformation;
        return this;
    }

    public BalayageBuilder addPostscriptumExternalScript(String name) {
        externalScriptNameList.add(name);
        return this;
    }

    public BalayageBuilder addPostscriptumScrutariExport(String name) {
        scrutariExportNameList.add(name);
        return this;
    }

    public BalayageBuilder addPostscriptumSqlExport(String name) {
        sqlExportNameList.add(name);
        return this;
    }

    public BalayageBuilder setDefaultLangOption(short defaultLangOption) {
        this.defaultLangOption = defaultLangOption;
        return this;
    }

    public BalayageBuilder setLangContext(LangContext langContext) {
        this.langContext = langContext;
        return this;
    }

    public BalayageBuilder setSiteMapOption(SiteMapOption siteMapOption) {
        this.siteMapOption = siteMapOption;
        return this;
    }

    public BalayageMode addBalayageMode(String name) {
        InternalBalayageMode ibe = modeMap.get(name);
        if (ibe == null) {
            ibe = new InternalBalayageMode(name);
            modeMap.put(name, ibe);
        }
        return ibe;
    }

    public BalayageBuilder setWithCurrentDateLimit(BalayageMode balayageMode, boolean b) {
        ((InternalBalayageMode) balayageMode).setWithCurrentDateLimit(b);
        return this;
    }

    public BalayageUnitBuilder addBalayageUnitBuilder(short type) {
        BalayageUnitBuilder unitBuilder = new BalayageUnitBuilder(type);
        unitList.add(unitBuilder);
        return unitBuilder;
    }

    public BalayageBuilder addIncludeFragment(String key, NodeList nodeList) {
        includeCatalogBuilder.addIncludeFragment(key, nodeList);
        return this;
    }

    public Balayage toBalayage() {
        List<FicheQuery> globalFicheQueryList = SelectionUtils.wrap(globalFicheQuery.toArray(new FicheQuery[globalFicheQuery.size()]));
        List<BalayageMode> balayageModeList;
        if (modeMap.isEmpty()) {
            balayageModeList = BalayageUtils.EMPTY_MODELIST;
        } else {
            balayageModeList = BalayageUtils.wrap(modeMap.values().toArray(new BalayageMode[modeMap.size()]));
        }
        List<BalayageUnit> balayageUnitList = BalayageUtils.EMPTY_UNITLIST;
        int unitLength = unitList.size();
        if (unitLength > 0) {
            BalayageUnit[] balayageUnitArray = new BalayageUnit[unitLength];
            for (int i = 0; i < unitLength; i++) {
                balayageUnitArray[i] = unitList.get(i).toBalayageUnit();
            }
            balayageUnitList = BalayageUtils.wrap(balayageUnitArray);
        }
        DocumentFragmentHolder includeCatalog = includeCatalogBuilder.toIncludeCatalog();
        Balayage.Postscriptum postscriptum = toPostscriptum();
        return new InternalBalayage(name, defaultLangOption, langContext, withTransformation, rootName, path, globalFicheQueryList, balayageModeList, balayageUnitList, siteMapOption, postscriptum, includeCatalog);
    }

    private Balayage.Postscriptum toPostscriptum() {
        return new InternalPostscriptum(StringUtils.toList(scrutariExportNameList), StringUtils.toList(sqlExportNameList), StringUtils.toList(externalScriptNameList));
    }

    public static BalayageBuilder init(String name) {
        return new BalayageBuilder(name);
    }


    private static class InternalBalayage implements Balayage {

        private final String name;
        private final short defaultLangOption;
        private final LangContext langContext;
        private final boolean withTransformation;
        private final String rootName;
        private final String path;
        private final List<FicheQuery> globalFicheQueryList;
        private final List<BalayageMode> balayageModeList;
        private final List<BalayageUnit> balayageUnitList;
        private final SiteMapOption siteMapOption;
        private final Balayage.Postscriptum postscriptum;
        private final DocumentFragmentHolder includeCatalog;

        private InternalBalayage(String name, short defaultLangOption, LangContext langContext, boolean withTransformation,
                String rootName, String path, List<FicheQuery> globalFicheQueryList, List<BalayageMode> balayageModeList, List<BalayageUnit> balayageUnitList,
                SiteMapOption siteMapOption, Balayage.Postscriptum postscriptum, DocumentFragmentHolder includeCatalog) {
            this.name = name;
            this.defaultLangOption = defaultLangOption;
            this.langContext = langContext;
            this.withTransformation = withTransformation;
            this.rootName = rootName;
            this.path = path;
            this.globalFicheQueryList = globalFicheQueryList;
            this.balayageModeList = balayageModeList;
            this.balayageUnitList = balayageUnitList;
            this.siteMapOption = siteMapOption;
            this.postscriptum = postscriptum;
            this.includeCatalog = includeCatalog;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public LangContext getLangContext() {
            return langContext;
        }

        @Override
        public String getRootName() {
            return rootName;
        }

        @Override
        public String getPath() {
            return path;
        }

        @Override
        public boolean isWithTransformation() {
            return withTransformation;
        }

        @Override
        public List<BalayageUnit> getBalayageUnitList() {
            return balayageUnitList;
        }

        @Override
        public List<FicheQuery> getGlobalFicheQueryList() {
            return globalFicheQueryList;
        }

        @Override
        public List<BalayageMode> getBalayageModeList() {
            return balayageModeList;
        }

        @Override
        public short getDefaultLangOption() {
            return defaultLangOption;
        }

        @Override
        public SiteMapOption getSiteMapOption() {
            return siteMapOption;
        }

        @Override
        public Postscriptum getPostscriptum() {
            return postscriptum;
        }

        @Override
        public DocumentFragmentHolder getIncludeCatalog() {
            return includeCatalog;
        }

    }


    private static class InternalPostscriptum implements Balayage.Postscriptum {

        private final List<String> scrutariExportNameList;
        private final List<String> sqlExportNameList;
        private final List<String> externalScriptNameList;

        private InternalPostscriptum(List<String> scrutariExportNameList, List<String> sqlExportNameList, List<String> externalScriptNameList) {
            this.scrutariExportNameList = scrutariExportNameList;
            this.sqlExportNameList = sqlExportNameList;
            this.externalScriptNameList = externalScriptNameList;
        }

        @Override
        public List<String> getScrutariExportNameList() {
            return scrutariExportNameList;
        }

        @Override
        public List<String> getSqlExportNameList() {
            return sqlExportNameList;
        }

        @Override
        public List<String> getExternalScriptNameList() {
            return externalScriptNameList;
        }


    }


    private static class InternalBalayageMode implements BalayageMode {

        private final String name;
        private boolean withCurrentDateLimit = false;

        InternalBalayageMode(String name) {
            this.name = name;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public boolean isWithCurrentDateLimit() {
            return withCurrentDateLimit;
        }

        private void setWithCurrentDateLimit(boolean withCurrentDateLimit) {
            this.withCurrentDateLimit = withCurrentDateLimit;
        }

    }


    private static class IncludeCatalogBuilder {

        private final Document document = DOMUtils.newDocument();
        private final Map<String, DocumentFragment> finalFragmentMap = new HashMap<String, DocumentFragment>();
        private final Map<String, DocumentFragmentBuilder> unresolvedFragmentMap = new HashMap<String, DocumentFragmentBuilder>();

        private IncludeCatalogBuilder() {
        }

        public DocumentFragmentHolder toIncludeCatalog() {
            resolve();
            return new InternalIncludeCatalog(finalFragmentMap);
        }

        private void resolve() {
            if (unresolvedFragmentMap.isEmpty()) {
                return;
            }
            boolean oneDone = false;
            for (Iterator<DocumentFragmentBuilder> it = unresolvedFragmentMap.values().iterator(); it.hasNext();) {
                DocumentFragmentBuilder documentFragmentBuilder = it.next();
                short result = documentFragmentBuilder.resolve(finalFragmentMap);
                switch (result) {
                    case RESOLVED_ALL:
                        it.remove();
                        oneDone = true;
                        break;
                    case RESOLVED_SOME:
                        oneDone = true;
                        break;
                    case RESOLVED_NONE:

                }
            }
            if (oneDone) {
                if (unresolvedFragmentMap.isEmpty()) {
                    return;
                } else {
                    resolve();
                }
            } else {
                //@todo avertissement toutes les inclusions n'ont pas été résolues
                return;
            }
        }

        private void addIncludeFragment(String key, NodeList nodeList) {
            if (finalFragmentMap.containsKey(key)) {
                finalFragmentMap.remove(key);
            }
            if (unresolvedFragmentMap.containsKey(key)) {
                unresolvedFragmentMap.remove(key);
            }
            DocumentFragment documentFragment = document.createDocumentFragment();
            int count = nodeList.getLength();
            for (int i = 0; i < count; i++) {
                Node node = nodeList.item(i);
                Node importNode = document.importNode(node, true);
                documentFragment.appendChild(importNode);
            }
            DocumentFragmentBuilder documentFragmentBuilder = new DocumentFragmentBuilder(key, documentFragment);
            if (documentFragmentBuilder.hasIncludeElement()) {
                unresolvedFragmentMap.put(key, documentFragmentBuilder);
            } else {
                finalFragmentMap.put(key, documentFragment);
            }
        }

    }


    private static class DocumentFragmentBuilder {

        private final String key;
        private final DocumentFragment documentFragment;
        private final List<Element> elementList = new LinkedList<Element>();

        private DocumentFragmentBuilder(String key, DocumentFragment documentFragment) {
            this.key = key;
            this.documentFragment = documentFragment;
            NodeList list = documentFragment.getChildNodes();
            int count = list.getLength();
            for (int i = 0; i < count; i++) {
                Node node = list.item(i);
                if (node instanceof Element) {
                    Element element = (Element) node;
                    if (element.getTagName().equals("include")) {
                        addIncludeElement(element);
                    } else {
                        NodeList includeChildren = element.getElementsByTagName("include");
                        int includeCount = includeChildren.getLength();
                        for (int j = 0; j < includeCount; j++) {
                            addIncludeElement((Element) includeChildren.item(j));
                        }
                    }
                }
            }
        }

        private short resolve(Map<String, DocumentFragment> finalFragmentMap) {
            boolean oneDone = false;
            for (Iterator<Element> it = elementList.iterator(); it.hasNext();) {
                Element element = it.next();
                String fragmentKey = element.getAttribute("fragment-key");
                DocumentFragment finalFragment = finalFragmentMap.get(fragmentKey);
                if (finalFragment != null) {
                    finalFragment = (DocumentFragment) finalFragment.cloneNode(true);
                    element.getParentNode().replaceChild(finalFragment, element);
                    oneDone = true;
                    it.remove();
                }
            }
            if (oneDone) {
                if (elementList.isEmpty()) {
                    finalFragmentMap.put(key, documentFragment);
                    return RESOLVED_ALL;
                } else {
                    return RESOLVED_SOME;
                }
            } else {
                return RESOLVED_NONE;
            }
        }

        private boolean hasIncludeElement() {
            return (elementList.size() > 0);
        }

        private void addIncludeElement(Element element) {
            NodeList includeChildren = element.getElementsByTagName("include");
            if (includeChildren.getLength() == 0) {
                String fragmentKey = element.getAttribute("fragment-key");
                if (fragmentKey.length() > 0) {
                    elementList.add(element);
                }
            } else {
                //@todo : avertissement, un élément include ne doit pas contenir un autre élément include
            }
        }

    }


    private static class InternalIncludeCatalog implements DocumentFragmentHolder {

        private final Map<String, DocumentFragment> fragmentMap;

        private InternalIncludeCatalog(Map<String, DocumentFragment> fragmentMap) {
            this.fragmentMap = fragmentMap;
        }

        @Override
        public DocumentFragment getFragment(String fragmentKey) {
            return fragmentMap.get(fragmentKey);
        }

    }

}
